@@ -37,11 +37,14 @@
 #include "ivs.h"
 
 
-const int CURSOR_UPDATE_TIME = 30;
+const int CURSOR_UPDATE_TIME = 35;
+
+int demoServer::s_numOfInstances = 0;
 
 
-demoServer::demoServer( IVS * _ivs_conn, int _quality, quint16 _port ) :
-	QTcpServer(),
+demoServer::demoServer( IVS * _ivs_conn, int _quality, quint16 _port,
+							QTcpSocket * _parent ) :
+	QTcpServer( _parent ),
 	m_conn( new ivsConnection(
 			QHostAddress( QHostAddress::LocalHost ).toString() +
 					":" + QString::number(
@@ -52,6 +55,8 @@ demoServer::demoServer( IVS * _ivs_conn, int _quality, quint16 _port ) :
 				_ivs_conn->runningInSeparateProcess() ) ),
 	m_updaterThread( new updaterThread( m_conn ) )
 {
+	++s_numOfInstances;
+
 	if( listen( QHostAddress::Any, _port ) == FALSE )
 	{
 		qCritical( "demoServer::demoServer(): "
@@ -61,6 +66,7 @@ demoServer::demoServer( IVS * _ivs_conn, int _quality, quint16 _port ) :
 
 	m_updaterThread->start(/* QThread::HighPriority*/ );
 
+	checkForCursorMovement();
 }
 
 
@@ -68,6 +74,13 @@ demoServer::demoServer( IVS * _ivs_conn, int _quality, quint16 _port ) :
 
 demoServer::~demoServer()
 {
+	QList<demoServerClient *> l;
+	while( !( l = findChildren<demoServerClient *>() ).isEmpty() )
+	{
+		delete l.front();
+	}
+	--s_numOfInstances;
+
 	delete m_updaterThread;
 	delete m_conn;
 }
@@ -75,9 +88,24 @@ demoServer::~demoServer()
 
 
 
+void demoServer::checkForCursorMovement( void )
+{
+	m_cursorLock.lockForWrite();
+	if( m_cursorPos != QCursor::pos() )
+	{
+		m_cursorPos = QCursor::pos();
+	}
+	m_cursorLock.unlock();
+	QTimer::singleShot( CURSOR_UPDATE_TIME, this,
+					SLOT( checkForCursorMovement() ) );
+}
+
+
+
+
 void demoServer::incomingConnection( int _sd )
 {
-	new demoServerClient( _sd, m_conn );
+	new demoServerClient( _sd, m_conn, this );
 }
 
 
@@ -119,6 +147,7 @@ void demoServer::updaterThread::run( void )
 		msleep( 20 );
 		m_conn->handleServerMessages( ( i = ( i + 1 ) % 2 ) == 0 );
 	}
+	m_conn->gracefulClose();
 }
 
 
@@ -131,13 +160,13 @@ void demoServer::updaterThread::run( void )
 
 
 
-demoServerClient::demoServerClient( int _sd, const ivsConnection * _conn ) :
-	QThread(),
+demoServerClient::demoServerClient( int _sd, const ivsConnection * _conn,
+							demoServer * _parent ) :
+	QThread( _parent ),
+	m_ds( _parent ),
 	m_dataMutex(),
 	m_changedRegion(),
-	m_lastCursorPos(),
 	m_cursorShapeChanged( TRUE ),
-	m_cursorPosChanged( FALSE ),
 	m_socketDescriptor( _sd ),
 	m_sock( NULL ),
 	m_conn( _conn ),
@@ -147,8 +176,6 @@ demoServerClient::demoServerClient( int _sd, const ivsConnection * _conn ) :
 			    		( sizeof( lzo_align_t ) - 1 ) ) /
 				 		sizeof( lzo_align_t ) ) ] )
 {
-	QTimer::singleShot( CURSOR_UPDATE_TIME, this,
-					SLOT( checkForCursorMovement() ) );
 	start();
 }
 
@@ -183,28 +210,13 @@ void demoServerClient::updateCursorShape( void )
 
 
 
-void demoServerClient::checkForCursorMovement( void )
-{
-	m_dataMutex.lock();
-	if( m_lastCursorPos != QCursor::pos() )
-	{
-		m_lastCursorPos = QCursor::pos();
-		m_cursorPosChanged = TRUE;
-	}
-	m_dataMutex.unlock();
-	QTimer::singleShot( CURSOR_UPDATE_TIME, this,
-					SLOT( checkForCursorMovement() ) );
-}
-
-
-
-
 void demoServerClient::moveCursor( void )
 {
-	m_dataMutex.lock();
-	if( m_cursorPosChanged )
+	QPoint p = m_ds->cursorPos();
+	if( p != m_lastCursorPos )
 	{
-		m_cursorPosChanged = FALSE;
+		m_dataMutex.lock();
+		m_lastCursorPos = p;
 		const rfbFramebufferUpdateMsg m =
 		{
 			rfbFramebufferUpdate,
@@ -229,9 +241,9 @@ void demoServerClient::moveCursor( void )
 		} ;
 
 		m_sock->write( (const char *) &rh, sizeof( rh ) );
-		m_sock->flush();
+		m_sock->waitForBytesWritten();
+		m_dataMutex.unlock();
 	}
-	m_dataMutex.unlock();
 }
 
 
@@ -300,8 +312,8 @@ void demoServerClient::processClient( void )
 			const QImage & i = m_conn->screen();
 			italcRectEncodingHeader hdr = { 0, 0, 0 } ;
 
-			const Q_UINT16 w = it->width();
-			const Q_UINT16 h = it->height();
+			const int w = it->width();
+			const int h = it->height();
 
 			// we only compress if it's enough data, otherwise
 			// there's too much overhead
@@ -314,11 +326,11 @@ void demoServerClient::processClient( void )
 	Q_UINT8 rle_sub = 1;
 	Q_UINT8 * out = new Q_UINT8[w * h * sizeof( QRgb )+16];
 	Q_UINT8 * out_ptr = out;
-	for( Q_UINT16 y = it->y(); y < it->y() + h; ++y )
+	for( int y = it->y(); y < it->y() + h; ++y )
 	{
 		const QRgb * data = ( (const QRgb *) i.scanLine( y ) ) +
 									it->x();
-		for( Q_UINT16 x = 0; x < w; ++x )
+		for( int x = 0; x < w; ++x )
 		{
 			if( data[x] != last_pix || rle_cnt > 254 )
 			{
@@ -339,8 +351,8 @@ void demoServerClient::processClient( void )
 	*( (QRgb *) out_ptr ) = last_pix;
 	*( out_ptr + 3 ) = rle_cnt;
 	out_ptr += 4;
-
 	hdr.bytesRLE = out_ptr - out;
+//printf("%d %d\n",hdr.bytesRLE, w*h*sizeof(QRgb)+16);
 	lzo_uint bytes_lzo = hdr.bytesRLE + hdr.bytesRLE / 16 + 67;
 	Q_UINT8 * comp = new Q_UINT8[bytes_lzo];
 	lzo1x_1_compress( (const unsigned char *) out, (lzo_uint) hdr.bytesRLE,
@@ -361,11 +373,12 @@ void demoServerClient::processClient( void )
 	if( m_otherEndianess )
 	{
 		Q_UINT32 * buf = new Q_UINT32[w];
-		for( Q_UINT16 y = 0; y < h; ++y )
+		for( int y = 0; y < h; ++y )
 		{
-			const QRgb * src = (const QRgb *) i.scanLine( it->y() + y ) +
-										it->x();
-			for( Q_UINT16 x = 0; x < w; ++x, ++src )
+			const QRgb * src = (const QRgb *) i.scanLine(
+								it->y() + y ) +
+									it->x();
+			for( int x = 0; x < w; ++x, ++src )
 			{
 				buf[x] = swap32( *src );
 			}
@@ -375,11 +388,12 @@ void demoServerClient::processClient( void )
 	}
 	else
 	{
-		for( Q_UINT16 y = 0; y < h; ++y )
+		for( int y = 0; y < h; ++y )
 		{
 			m_sock->write( (const char *)
-				( (const QRgb *) i.scanLine( it->y() + y ) + it->x() ),
-								w * sizeof( QRgb ) );
+				( (const QRgb *) i.scanLine( it->y() + y ) +
+								it->x() ),
+							w * sizeof( QRgb ) );
 		}
 	}
 			}
@@ -413,11 +427,9 @@ void demoServerClient::processClient( void )
 		m_changedRegion = QRegion();
 		m_cursorShapeChanged = FALSE;
 	}
-	//m_sock->waitForBytesWritten();
-	m_sock->flush();
+	m_sock->waitForBytesWritten();
 
 	m_dataMutex.unlock();
-	//QTimer::singleShot( 40, this, SLOT( processClient() ) );
 }
 
 
@@ -537,9 +549,11 @@ void demoServerClient::run( void )
 	// working
 
 	connect( m_conn, SIGNAL( cursorShapeChanged() ),
-				this, SLOT( updateCursorShape() ) );
+			this, SLOT( updateCursorShape() ),
+							Qt::QueuedConnection );
 	connect( m_conn, SIGNAL( regionUpdated( const QRegion & ) ),
-			this, SLOT( updateRegion( const QRegion & ) ) );
+			this, SLOT( updateRegion( const QRegion & ) ),
+							Qt::QueuedConnection );
 
 	ml.unlock();
 
@@ -547,17 +561,17 @@ void demoServerClient::run( void )
 	updateRegion( m_conn->screen().rect() );
 
 	//connect( m_sock, SIGNAL( readyRead() ), this, SLOT( processClient() ) );
-	connect( m_sock, SIGNAL( disconnected() ), this,
-							SLOT( deleteLater() ) );
+	connect( m_sock, SIGNAL( disconnected() ),
+						this, SLOT( deleteLater() ) );
 
 	QTimer t;
-	connect( &t, SIGNAL( timeout() ), this, SLOT( moveCursor() ),
-							Qt::DirectConnection );
+	connect( &t, SIGNAL( timeout() ),
+			this, SLOT( moveCursor() ), Qt::DirectConnection );
 	t.start( CURSOR_UPDATE_TIME );
 	QTimer t2;
-	connect( &t2, SIGNAL( timeout() ), this, SLOT( processClient() ),
-							Qt::DirectConnection );
-	t2.start( 50 );
+	connect( &t2, SIGNAL( timeout() ),
+			this, SLOT( processClient() ), Qt::DirectConnection );
+	t2.start( 2*CURSOR_UPDATE_TIME );
 	//moveCursor();
 	//processClient();
 	// now run our own event-loop for optimal scheduling
