@@ -29,6 +29,7 @@
 
 #include <QtCore/QMutex>
 #include <QtCore/QPair>
+#include <QtCore/QReadWriteLock>
 #include <QtCore/QThread>
 #include <QtNetwork/QTcpServer>
 
@@ -43,14 +44,40 @@ class demoServer : public QTcpServer
 {
 	Q_OBJECT
 public:
-	demoServer( IVS * _ivs_conn, int _quality = 0, quint16 _port = 0 );
+	demoServer( IVS * _ivs_conn, int _quality, quint16 _port,
+							QTcpSocket * _parent );
 	virtual ~demoServer();
 
+	inline QPoint cursorPos( void )
+	{
+		m_cursorLock.lockForRead();
+		QPoint p = m_cursorPos;
+		m_cursorLock.unlock();
+		return( p );
+	}
+
+	static int numOfInstances( void )
+	{
+		return( s_numOfInstances );
+	}
+
+
+private slots:
+	// checks whether cursor was moved and sets according flags and
+	// variables used by moveCursor() - connection has to be done in
+	// GUI-thread as we're calling QCursor::pos() which at least under X11
+	// must not be called from another thread than the GUI-thread
+	void checkForCursorMovement( void );
+
 
 private:
 	virtual void incomingConnection( int _sd );
 
+	static int s_numOfInstances;
+
 	ivsConnection * m_conn;
+	QReadWriteLock m_cursorLock;
+	QPoint m_cursorPos;
 
 	// this thread is just responsible for updating IVS-connection's screen
 	class updaterThread : public QThread
@@ -79,7 +106,8 @@ class demoServerClient : public QThread
 {
 	Q_OBJECT
 public:
-	demoServerClient( int _sd, const ivsConnection * _conn );
+	demoServerClient( int _sd, const ivsConnection * _conn,
+							demoServer * _parent );
 	virtual ~demoServerClient();
 
 
@@ -93,12 +121,6 @@ private slots:
 	// called whenever ivsConnection::cursorShapeChanged() is emitted
 	void updateCursorShape( void );
 
-	// checks whether cursor was moved and sets according flags and
-	// variables used by moveCursor() - connection has to be done in
-	// GUI-thread as we're calling QCursor::pos() which at least under X11
-	// must not be called from another thread than the GUI-thread
-	void checkForCursorMovement( void );
-
 	// called regularly for sending pointer-movement-events detected by
 	// checkForCursorMovement() to clients - connection has to be done
 	// in demoServerClient-thread-context as we're writing to socket
@@ -114,11 +136,11 @@ private:
 	// event-loop of thread
 	virtual void run( void );
 
+	demoServer * m_ds;
 	QMutex m_dataMutex;
 	QRegion m_changedRegion;
 	QPoint m_lastCursorPos;
-	bool m_cursorShapeChanged;
-	bool m_cursorPosChanged;
+	volatile bool m_cursorShapeChanged;
 
 	int m_socketDescriptor;
 	QTcpSocket * m_sock;
