@@ -23,6 +23,18 @@
  */
 
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+#ifdef BUILD_WIN32
+
+#define _WIN32_WINNT 0x0501
+#include <windows.h>
+#include <psapi.h>
+#endif
+
+
 #include <QtCore/QCoreApplication>
 #include <QtCore/QDir>
 #include <QtCore/QProcess>
@@ -44,7 +56,6 @@
 #include "demo_server.h"
 #include "ica_main.h"
 
-
 static isdServer * __isd_server = NULL;
 
 QStringList isdServer::s_allowedDemoClients;
@@ -53,13 +64,13 @@ QStringList isdServer::s_allowedDemoClients;
 
 isdServer::isdServer( const quint16 _ivs_port, int _argc, char * * _argv ) :
 	QTcpServer(),
+	m_readyReadMapper( this ),
 	m_ivs( NULL ),
 	m_demoClient( NULL ),
-	m_demoServer( NULL ),
 	m_lockWidget( NULL )
 {
 	if( __isd_server ||
-			listen( QHostAddress::LocalHost, __isd_port ) == FALSE )
+			listen( QHostAddress::Any, __isd_port ) == FALSE )
 	{
 		// uh oh, already an ISD running or port isn't available...
 		qCritical( "isdServer::isdServer(...): "
@@ -77,6 +88,9 @@ isdServer::isdServer( const quint16 _ivs_port, int _argc, char * * _argv ) :
 	connect( this, SIGNAL( newConnection() ),
 			this, SLOT( acceptNewConnection() ) );
 
+	connect( &m_readyReadMapper, SIGNAL( mapped( QObject * ) ),
+			this, SLOT( processClient( QObject * ) ) );
+
 	QTimer * t = new QTimer( this );
 	connect( t, SIGNAL( timeout() ), this,
 					SLOT( checkForPendingActions() ) );
@@ -98,7 +112,6 @@ isdServer::isdServer( const quint16 _ivs_port, int _argc, char * * _argv ) :
 isdServer::~isdServer()
 {
 	delete m_ivs;
-	delete m_demoServer;
 	delete m_lockWidget;
 	__isd_server = NULL;
 }
@@ -146,17 +159,107 @@ int isdServer::processClient( socketDispatcher _sd, void * _user )
 			const QString cmds = msg_in.arg( "cmds" ).toString();
 			if( !cmds.isEmpty() )
 			{
+#ifdef BUILD_WIN32
+	// run process as the user which is logged on
+	DWORD aProcesses[1024], cbNeeded;
+
+	if( !EnumProcesses( aProcesses, sizeof( aProcesses ), &cbNeeded ) )
+	{
+		break;
+	}
+
+	DWORD cProcesses = cbNeeded / sizeof(DWORD);
+
+	for( DWORD i = 0; i < cProcesses; i++ )
+	{
+		HANDLE hProcess = OpenProcess( PROCESS_ALL_ACCESS,
+							false, aProcesses[i] );
+		HMODULE hMod;
+		if( hProcess == NULL ||
+			!EnumProcessModules( hProcess, &hMod, sizeof( hMod ),
+								&cbNeeded ) )
+	        {
+			continue;
+		}
+
+		TCHAR szProcessName[MAX_PATH];
+		GetModuleBaseName( hProcess, hMod, szProcessName, 
+                       		  sizeof( szProcessName ) / sizeof( TCHAR) );
+		for( TCHAR * ptr = szProcessName; *ptr; ++ptr )
+		{
+			*ptr = tolower( *ptr );
+		}
+
+		if( strcmp( szProcessName, "explorer.exe" ) )
+		{
+			CloseHandle( hProcess );
+			continue;
+		}
+	
+		HANDLE hToken;
+		OpenProcessToken( hProcess, MAXIMUM_ALLOWED, &hToken );
+		ImpersonateLoggedOnUser( hToken );
+
+		STARTUPINFO si;
+		PROCESS_INFORMATION pi;
+		ZeroMemory( &si, sizeof( STARTUPINFO ) );
+		si.cb= sizeof( STARTUPINFO );
+		si.lpDesktop = (CHAR *) "winsta0\\default";
+		HANDLE hNewToken = NULL;
+
+		DuplicateTokenEx( hToken, MAXIMUM_ALLOWED, NULL,
+					SecurityImpersonation, TokenPrimary,
+								&hNewToken );
+
+		CreateProcessAsUser(
+				hNewToken,            // client's access token
+				NULL,              // file to execute
+				(CHAR *)cmds.toAscii().constData(),     // command line
+				NULL,              // pointer to process SECURITY_ATTRIBUTES
+				NULL,              // pointer to thread SECURITY_ATTRIBUTES
+				FALSE,             // handles are not inheritable
+				NORMAL_PRIORITY_CLASS | CREATE_NEW_CONSOLE,   // creation flags
+				NULL,              // pointer to new environment block 
+				NULL,              // name of current directory 
+				&si,               // pointer to STARTUPINFO structure
+				&pi                // receives information about new process
+				); 
+		CloseHandle( hNewToken );
+		RevertToSelf();
+		CloseHandle( hToken );
+		CloseHandle( hProcess );
+		break;
+	}
+#else
 				QProcess::startDetached( cmds );
+#endif
 			}
 			break;
 		}
 
 		case ISD::StartFullScreenDemo:
 		case ISD::StartWindowDemo:
-/*		case ISD::ViewRemoteDisplay:
-		case ISD::RemoteControlDisplay:*/
-			action = msg_in.arg( "ip" ).toString();
+		{
+			QString port = msg_in.arg( "port" ).toString();
+			if( port == "" )
+			{
+				port = "5858";
+			}
+			if( !port.contains( ':' ) )
+			{
+				const int MAX_HOST_LEN = 255;
+				char host[MAX_HOST_LEN+1];
+				_sd( host, MAX_HOST_LEN, SocketGetPeerAddress,
+									_user );
+				host[MAX_HOST_LEN] = 0;
+				action = host + QString( ":" ) + port;
+			}
+			else
+			{
+				action = port;
+			}
 			break;
+		}
 
 		case ISD::DisplayTextMessage:
 			action = msg_in.arg( "msg" ).toString();
@@ -217,34 +320,48 @@ int isdServer::processClient( socketDispatcher _sd, void * _user )
 		}
 
 		case ISD::DemoServer_Run:
-			// make sure any running demo-server is destroyed
-			delete m_demoServer;
-			// start demo-server on local IVS
-			m_demoServer = new demoServer( m_ivs,
-					msg_in.arg( "quality" ).toInt(),
-					msg_in.arg( "port" ).toInt() );
 			if( _sd == &qtcpsocketDispatcher )
 			{
+				// start demo-server on local IVS and make it
+				// child of our socket so that it automatically
+				// gets destroyed as soon as the socket is
+				// closed and thus destroyed
 				QTcpSocket * ts = static_cast<QTcpSocket *>(
 								_user );
-				ts->setProperty( "demoserver",
-					QVariant::fromValue( (QObject *)
-							m_demoServer ) );
+				new demoServer( m_ivs,
+					msg_in.arg( "quality" ).toInt(),
+					msg_in.arg( "port" ).toInt(), ts );
+			}
+			else
+			{
+				qCritical( "socket-dispatcher is not a "
+						"qtcpsocketDispatcher!\n" );
 			}
 			break;
 
-		case ISD::DemoServer_Stop:
-		{
-			const QList<QTcpSocket *> sockets =
-						findChildren<QTcpSocket *>();
-			foreach( QTcpSocket * _ts, sockets )
+		case ISD::HideTrayIcon:
+#ifdef SYSTEMTRAY_SUPPORT
+			if( _sd == &qtcpsocketDispatcher )
 			{
-				cleanupDestroyedConnection( _ts );
+				// start demo-server on local IVS and make it
+				// child of our socket so that it automatically
+				// gets destroyed as soon as the socket is
+				// closed and thus destroyed
+				QTcpSocket * ts = static_cast<QTcpSocket *>(
+								_user );
+				__systray_icon->hide();
+				connect( ts, SIGNAL( disconnected() ),
+					__systray_icon, SLOT( show() ) );
 			}
-			//delete m_demoServer;
-			m_demoServer = NULL;
+			else
+			{
+				qCritical( "socket-dispatcher is not a "
+						"qtcpsocketDispatcher!\n" );
+			}
+#endif
+			break;
+
 			break;
-		}
 
 		case ISD::DemoServer_AllowClient:
 			allowDemoClient( msg_in.arg( "client" ).toString() );
@@ -255,7 +372,7 @@ int isdServer::processClient( socketDispatcher _sd, void * _user )
 			break;
 
 		default:
-			qCritical( "isdServer::processClients(...): "
+			qCritical( "isdServer::processClient(...): "
 					"cmd %d not implemented!", cmd );
 			break;
 	}
@@ -322,7 +439,7 @@ bool isdServer::protocolInitialization( socketDevice & _sd,
 
 	const int MAX_HOST_LEN = 255;
 	char host[MAX_HOST_LEN+1];
-	_sd.sockDispatcher()( host, MAX_HOST_LEN, SocketGetIPBoundTo,
+	_sd.sockDispatcher()( host, MAX_HOST_LEN, SocketGetPeerAddress,
 								_sd.user() );
 	host[MAX_HOST_LEN] = 0;
 
@@ -355,7 +472,7 @@ bool isdServer::authSecTypeItalc( socketDispatcher _sd, void * _user,
 	// find out IP of host - needed at several places
 	const int MAX_HOST_LEN = 255;
 	char host[MAX_HOST_LEN+1];
-	_sd( host, MAX_HOST_LEN, SocketGetIPBoundTo, _user );
+	_sd( host, MAX_HOST_LEN, SocketGetPeerAddress, _user );
 	host[MAX_HOST_LEN] = 0;
 	static QStringList __denied_hosts, __allowed_hosts;
 
@@ -426,7 +543,7 @@ bool isdServer::authSecTypeItalc( socketDispatcher _sd, void * _user,
 				foreach( const QHostAddress a, addr )
 				{
 	if( allowed.contains( a.toString() ) ||
-			a.toString() == QHostAddress( QHostAddress::LocalHost ).toString() )
+		a.toString() == QHostAddress( QHostAddress::LocalHost ).toString() )
 					{
 						result = ItalcAuthOK;
 						break;
@@ -603,7 +720,7 @@ void isdServer::acceptNewConnection( void )
 	QTcpSocket * sock = nextPendingConnection();
 	socketDevice sd( qtcpsocketDispatcher, sock );
 
-	if( !protocolInitialization( sd, ItalcAuthLocalDSA) )
+	if( !protocolInitialization( sd, ItalcAuthLocalDSA ) )
 	{
 		delete sock;
 		return;
@@ -611,44 +728,22 @@ void isdServer::acceptNewConnection( void )
 
 	// now we're ready to start the normal interaction with the client,
 	// so make sure, we get informed about new requests
-	connect( sock, SIGNAL( disconnected() ), sock, SLOT( deleteLater() ) );
-	connect( sock, SIGNAL( readyRead() ), this, SLOT( processClients() ) );
-	connect( sock, SIGNAL( destroyed( QObject * ) ), this,
-			SLOT( cleanupDestroyedConnection( QObject * ) ) );
+	connect( sock, SIGNAL( readyRead() ),
+			&m_readyReadMapper, SLOT( map() ) );
+	connect( sock, SIGNAL( disconnected() ),
+			sock, SLOT( deleteLater() ) );
+	m_readyReadMapper.setMapping( sock, sock );
 }
 
 
 
 
-void isdServer::cleanupDestroyedConnection( QObject * _o )
+void isdServer::processClient( QObject * _sock )
 {
-	if( _o->property( "demoserver" ).isValid() )
+	QTcpSocket * sock = qobject_cast<QTcpSocket *>( _sock );
+	while( sock->bytesAvailable() > 0 )
 	{
-		demoServer * ds = dynamic_cast<demoServer *>(
-			_o->property( "demoserver" ).value<QObject *>() );
-		if( ds == m_demoServer )
-		{
-			m_demoServer = NULL;
-		}
-		delete ds;
-		_o->setProperty( "demoserver", QVariant() );
-	}
-}
-
-
-
-
-void isdServer::processClients( void )
-{
-	// check all open connections for available data
-	QList<QTcpSocket *> sockets = findChildren<QTcpSocket *>();
-	for( QList<QTcpSocket *>::iterator it = sockets.begin();
-						it != sockets.end(); ++it )
-	{
-		while( ( *it )->bytesAvailable() > 0 )
-		{
-			processClient( qtcpsocketDispatcher, *it );
-		}
+		processClient( qtcpsocketDispatcher, sock );
 	}
 }
 
@@ -685,12 +780,6 @@ void isdServer::checkForPendingActions( void )
 				displayTextMessage( data );
 				break;
 
-/*			case ISD::ViewRemoteDisplay:
-			case ISD::RemoteControlDisplay:
-				remoteControlDisplay( data,
-		( m_pendingActions.front().first == ISD::ViewRemoteDisplay ) );
-				break;
-*/
 			default:
 				qWarning( "isdServer::checkForPendingActions():"
 						" unhandled command %d",
@@ -719,7 +808,7 @@ void isdServer::startDemo( const QString & _master_host, bool _fullscreen )
 	// if a demo-server is started, it's likely that the demo was started
 	// on master-computer as well therefore we deny starting a demo on
 	// hosts on which a demo-server is running
-	if( m_demoServer )
+	if( demoServer::numOfInstances() > 0 )
 	{
 		return;
 	}
@@ -743,7 +832,7 @@ void isdServer::stopDemo( void )
 
 void isdServer::lockDisplay( void )
 {
-	if( m_demoServer )
+	if( demoServer::numOfInstances() )
 	{
 		return;
 	}
@@ -845,75 +934,6 @@ public:
 	{
 	}
 
-#if 0
-	bool handleServerMessage( Q_UINT8 _msg )
-	{
-		if( _msg != rfbItalcServiceResponse )
-		{
-			printf( "Unknown message type %d from server. Closing "
-				"connection. Will re-open it later.\n", _msg );
-			close();
-			return( FALSE );
-		}
-
-		Q_UINT8 cmd;
-		if( !readFromServer( (char *) &cmd, sizeof( cmd ) ) )
-		{
-			return( FALSE );
-		}
-		switch( cmd )
-		{
-			case ISD::UserInformation:
-			{
-				ISD::msg m( &socketDev() );
-				m.receive();
-				ISD::msg( &socketDev(), ISD::UserInformation ).
-						addArg( "username",
-							m.arg( "username" ) ).
-						addArg( "homedir",
-							m.arg( "homedir" ) ).
-									send();
-				break;
-			}
-
-/*			case ISD::DemoServer_PortInfo:
-			{
-				ISD::msg m( &socketDev() );
-				m.receive();
-				ISD::msg( &socketDev(),
-						ISD::DemoServer_PortInfo ).
-					addArg( "demoserverport",
-						m.arg( "demoserverport" ).
-							toInt() ).send();
-				break;
-			}*/
-
-			default:
-				printf( "Unknown server response %d\n",
-								(int) cmd );
-				return( FALSE );
-		}
-
-		return( TRUE );
-	}
-
-
-	void handleServerMessages( void )
-	{
-		printf("handle\n");
-		while( hasData() )
-		{
-			rfbServerToClientMsg msg;
-			if( !readFromServer( (char *) &msg,
-							sizeof( Q_UINT8 ) ) )
-			{
-				printf( "Reading message-type failed\n" );
-				continue;
-			}
-			handleServerMessage( msg.type );
-		}
-	}
-#endif
 
 	int processClient( socketDispatcher _sd, void * _user )
 	{
@@ -951,16 +971,21 @@ public:
 
 			case ISD::StartFullScreenDemo:
 			case ISD::StartWindowDemo:
-				startDemo( msg_in.arg( "ip" ).toString(),
+			{
+				const int MAX_HOST_LEN = 255;
+				char host[MAX_HOST_LEN+1];
+				_sd( host, MAX_HOST_LEN, SocketGetPeerAddress,
+									_user );
+				host[MAX_HOST_LEN] = 0;
+				QString port = msg_in.arg( "port" ).toString();
+				if( port == "" )
+				{
+					port = "5858";
+				}
+				startDemo( host + QString( ":" ) + port,
 					cmd == ISD::StartFullScreenDemo );
 				break;
-
-/*			case ISD::ViewRemoteDisplay:
-			case ISD::RemoteControlDisplay:
-				remoteControlDisplay( msg_in.arg( "ip" ).
-								toString(),
-						cmd == ISD::ViewRemoteDisplay );
-				break;*/
+			}
 
 			case ISD::DisplayTextMessage:
 				displayTextMessage( msg_in.arg( "msg" ).
@@ -1007,10 +1032,6 @@ public:
 						msg_in.arg( "port" ).toInt() );
 				break;
 
-			case ISD::DemoServer_Stop:
-				demoServerStop();
-				break;
-
 			case ISD::DemoServer_AllowClient:
 				demoServerAllowClient(
 					msg_in.arg( "client" ).toString() );
