@@ -3,6 +3,7 @@
 #include <rfb/rfbconfig.h>
 #include <rfb/keysym.h>
 
+#include "x11vnc/avahi.c"
 #include "x11vnc/pm.c"
 #include "x11vnc/rates.c"
 #include "x11vnc/cleanup.c"
@@ -42,16 +43,6 @@
 #include "x11vnc/macosxCGP.c"
 #include "x11vnc/macosxCGS.c"
 
-/*
-for x11vnc 0.8.5
-#include "x11vnc/avahi.c"
-#include "libvncclient/vncviewer.c"
-#include "libvncclient/cursor.c"
-#include "libvncclient/listen.c"
-#include "libvncclient/rfbproto.c"
-#include "libvncclient/sockets.c"
-*/
-
 #define db db2
 #include "x11vnc/uinput.c"
 #undef db
