@@ -227,6 +227,76 @@ static void blackout_tiles(void) {
 	}
 }
 
+static int did_xinerama_clip = 0;
+
+void check_xinerama_clip(void) {
+#if LIBVNCSERVER_HAVE_LIBXINERAMA
+	int n, k, i, ev, er, juse = -1;
+	int score[32], is = 0;
+	XineramaScreenInfo *x;
+
+	if (!clip_str || !dpy) {
+		return;
+	}
+	if (sscanf(clip_str, "xinerama%d", &k) == 1) {
+		;
+	} else if (sscanf(clip_str, "screen%d", &k) == 1) {
+		;
+	} else {
+		return;
+	}
+
+	free(clip_str);
+	clip_str = NULL;
+
+	if (! XineramaQueryExtension(dpy, &ev, &er)) {
+		return;
+	}
+	if (! XineramaIsActive(dpy)) {
+		return;
+	}
+	x = XineramaQueryScreens(dpy, &n);
+	if (k < 0 || k >= n) {
+		XFree_wr(x);
+		return;
+	}
+	for (i=0; i < n; i++) {
+		score[is++] = nabs(x[i].x_org) + nabs(x[i].y_org);
+		if (is >= 32) {
+			break;
+		}
+	}
+	for (i=0; i <= k; i++) {
+		int j, jmon, mon = -1, mox = -1;
+		for (j=0; j < is; j++) {
+			if (mon < 0 || score[j] < mon) {
+				mon = score[j];
+				jmon = j;
+			}
+			if (mox < 0 || score[j] > mox) {
+				mox = score[j];
+			}
+		}
+		juse = jmon;
+		score[juse] = mox+1+i;
+	}
+
+	if (juse >= 0 && juse < n) {
+		char str[64];
+		sprintf(str, "%dx%d+%d+%d", x[juse].width, x[juse].height,
+		    x[juse].x_org, x[juse].y_org);
+		clip_str = strdup(str);
+		did_xinerama_clip = 1;
+	} else {
+		clip_str = strdup("");
+	}
+	XFree_wr(x);
+	if (!quiet) {
+		rfbLog("set -clip to '%s' for xinerama%d\n", clip_str, k);
+	}
+#endif
+}
+
 static void initialize_xinerama (void) {
 #if !LIBVNCSERVER_HAVE_LIBXINERAMA
 	rfbLog("Xinerama: Library libXinerama is not available to determine\n");
@@ -260,6 +330,8 @@ static void initialize_xinerama (void) {
 		return;
 	} 
 	xinerama_present = 1;
+	rfbLog("\n");
+	rfbLog("Xinerama is present and active (e.g. multi-head).\n");
 
 	/* n.b. change to XineramaGetData() someday */
 	xineramas = XineramaQueryScreens(dpy, &n);
@@ -267,10 +339,18 @@ static void initialize_xinerama (void) {
 		rfbLog("Xinerama: number of sub-screens: %d\n", n);
 	}
 
+	if (! use_xwarppointer && ! got_noxwarppointer && n > 1) {
+		rfbLog("Xinerama: enabling -xwarppointer mode to try to correct\n");
+		rfbLog("Xinerama: mouse pointer motion. XTEST+XINERAMA bug.\n");
+		rfbLog("Xinerama: Use -noxwarppointer to force XTEST.\n");
+		use_xwarppointer = 1;
+	}
+
 	if (n == 1) {
 		if (verbose) {
 			rfbLog("Xinerama: no blackouts needed (only one"
 			    " sub-screen)\n");
+			rfbLog("\n");
 		}
 		XFree_wr(xineramas);
 		return;		/* must be OK w/o change */
@@ -295,12 +375,18 @@ static void initialize_xinerama (void) {
 	}
 	XFree_wr(xineramas);
 
+
 	if (sraRgnEmpty(black_region)) {
 		rfbLog("Xinerama: no blackouts needed (screen fills"
 		    " rectangle)\n");
+		rfbLog("\n");
 		sraRgnDestroy(black_region);
 		return;
 	}
+	if (did_xinerama_clip) {
+		rfbLog("Xinerama: no blackouts due to -clip xinerama.\n");
+		return;
+	}
 
 	/* max len is 10000x10000+10000+10000 (23 chars) per geometry */
 	rcnt = (int) sraRgnCountRects(black_region);
@@ -332,6 +418,7 @@ static void initialize_xinerama (void) {
 	}
 	sraRgnReleaseIterator(iter);
 	initialize_blackouts(bstr);
+	rfbLog("\n");
 
 	free(bstr);
 	free(tstr);
@@ -370,11 +457,17 @@ void push_sleep(int n) {
  * try to forcefully push a black screen to all connected clients
  */
 void push_black_screen(int n) {
+	int Lx = dpy_x, Ly = dpy_y;
 	if (!screen) {
 		return;
 	}
-	zero_fb(0, 0, dpy_x, dpy_y);
-	mark_rect_as_modified(0, 0, dpy_x, dpy_y, 0);
+#ifndef NO_NCACHE
+	if (ncache > 0) {
+		Ly = dpy_y * (1+ncache);
+	}
+#endif
+	zero_fb(0, 0, Lx, Ly);
+	mark_rect_as_modified(0, 0, Lx, Ly, 0);
 	push_sleep(n);
 }
 
@@ -394,13 +487,22 @@ void refresh_screen(int push) {
  */
 void zero_fb(int x1, int y1, int x2, int y2) {
 	int pixelsize = bpp/8;
-	int line, fill = 0;
+	int line, fill = 0, yfac = 1;
 	char *dst;
+
+#ifndef NO_NCACHE
+	if (ncache > 0) {
+		yfac = 1+ncache;
+		if (ncache_xrootpmap) {
+			yfac++;
+		}
+	}
+#endif
 	
 	if (x1 < 0 || x2 <= x1 || x2 > dpy_x) {
 		return;
 	}
-	if (y1 < 0 || y2 <= y1 || y2 > dpy_y) {
+	if (y1 < 0 || y2 <= y1 || y2 > yfac * dpy_y) {
 		return;
 	}
 	if (! main_fb) {
