@@ -30,6 +30,7 @@
 #include <QtGui/QPainter>
 #include <QtGui/QPixmap>
 #include <QtGui/QScrollArea>
+#include <QtGui/QMessageBox>
 
 
 #include "main_window.h"
@@ -47,31 +48,11 @@
 
 const QSize DEFAULT_CLIENT_SIZE( 256, 192 );
 const int DECO_WIDTH = 4;
-const int TITLE_HEIGHT = 20;
+const int TITLE_HEIGHT = 23;
 const QPoint CONTENT_OFFSET( DECO_WIDTH, DECO_WIDTH + TITLE_HEIGHT ); 
 const QSize CONTENT_SIZE_SUB( 2*DECO_WIDTH, 2*DECO_WIDTH + TITLE_HEIGHT ); 
 
 
-const client::clientCommand client::s_commands[client::Cmd_CmdCount] =
-{
-
-	{ Cmd_None,		NULL,				(char *) "",								(char *) "",			FALSE	},
-	{ Cmd_Reload,		&client::reload,		(char *) "",								(char *) "", 			TRUE	},
-	{ Cmd_ViewLive,		&client::viewLive,		(char *) QT_TRANSLATE_NOOP( "client", "View live" ),			(char *) "viewmag.png",		FALSE	},
-	{ Cmd_RemoteControl,	&client::remoteControl,		(char *) QT_TRANSLATE_NOOP( "client", "Remote control" ),		(char *) "remote_control.png",	FALSE	},
-	{ Cmd_ClientDemo,	&client::clientDemo,		(char *) QT_TRANSLATE_NOOP( "client", "Let student show demo" ),		(char *) "client_demo.png",	FALSE	},
-	{ Cmd_SendTextMessage,	&client::sendTextMessage,	(char *) QT_TRANSLATE_NOOP( "client", "Send text message" ),		(char *) "text_message.png",	TRUE	},
-	{ Cmd_LogonUser,	&client::logonUser,		(char *) QT_TRANSLATE_NOOP( "client", "Logon user" ),			(char *) "remotelogon.png",	FALSE	},
-	{ Cmd_LogoutUser,	&client::logoutUser,		(char *) QT_TRANSLATE_NOOP( "client", "Logout user" ),			(char *) "logout.png",		TRUE	},
-	{ Cmd_Snapshot,		&client::snapshot,		(char *) QT_TRANSLATE_NOOP( "client", "Take a snapshot" ),		(char *) "snapshot.png", 	TRUE	},
-	{ Cmd_PowerOn,		&client::powerOn,		(char *) QT_TRANSLATE_NOOP( "client", "Power on" ),			(char *) "power_on.png",		FALSE	},
-	{ Cmd_Reboot,		&client::reboot,		(char *) QT_TRANSLATE_NOOP( "client", "Reboot" ),			(char *) "reboot.png",		FALSE	},
-	{ Cmd_PowerDown,	&client::powerDown,		(char *) QT_TRANSLATE_NOOP( "client", "Power down" ),			(char *) "power_off.png",	FALSE	},
-	{ Cmd_ExecCmds,		&client::execCmds,		(char *) QT_TRANSLATE_NOOP( "client", "Execute commands" ),		(char *) "run.png", 		FALSE	}
-
-} ;
-
-
 // resolve static symbols...
 QHash<int, client *> client::s_clientIDs;
 
@@ -138,9 +119,353 @@ private:
 
 
 
+clientAction::clientAction( type _type, QObject * _parent, int _flags ) : 
+	QAction( _parent ),
+	m_type( _type ),
+	m_flags( _flags )
+{
+}
 
-client::client( const QString & _local_ip,
-		const QString & _mac, const QString & _name,
+
+
+
+clientAction::clientAction( type _type, const QIcon & _icon, const QString & _text,
+				QObject * _parent, int _flags ) : 
+	QAction( _icon, _text, _parent ),
+	m_type( _type ),
+	m_flags( _flags )
+{
+}
+
+
+
+
+void clientAction::process( QVector<client *> _clients, targetGroup _target )
+{
+	switch ( m_type )
+	{
+		case Overview:
+			foreach( client * cl, _clients )
+			{
+				cl->changeMode( client::Mode_Overview );
+			}
+			break;
+			
+		case FullscreenDemo:
+			foreach( client * cl, _clients )
+			{
+				cl->changeMode( client::Mode_FullscreenDemo );
+			}
+			break;
+			
+		case WindowDemo:
+			foreach( client * cl, _clients )
+			{
+				cl->changeMode( client::Mode_WindowDemo );
+			}
+			break;
+			
+		case Locked:
+			foreach( client * cl, _clients )
+			{
+				cl->changeMode( client::Mode_Locked );
+			}
+			break;
+			
+		case ViewLive:
+			foreach( client * cl, _clients )
+			{
+				cl->viewLive();
+			}
+			break;
+			
+		case RemoteControl:
+			foreach( client * cl, _clients )
+			{
+				cl->remoteControl();
+			}
+			break;
+			
+		case ClientDemo:
+			foreach( client * cl, _clients )
+			{
+				cl->clientDemo();
+			}
+			break;
+			
+		case SendTextMessage:
+			{
+				QString msg;
+				textMessageDialog tmd( msg );
+
+				if( tmd.exec() == QDialog::Accepted &&
+					!msg.isEmpty() )
+				{ 
+					foreach( client * cl, _clients )
+					{
+						cl->sendTextMessage( msg );
+					}
+				}
+			}
+
+			break;
+
+		case LogonUser:
+			{
+				remoteLogonDialog mld;
+
+				if( mld.exec() == QDialog::Accepted &&
+					!mld.userName().isEmpty() &&
+					!mld.password().isEmpty() )
+				{
+					foreach( client * cl, _clients )
+					{
+						cl->logonUser( mld.userName(),
+							mld.password(), mld.domain() );
+					}
+				}
+
+			}
+			break;
+
+		case LogoutUser:
+			if ( confirmLogout( _target ) )
+			{
+				foreach( client * cl, _clients )
+				{
+					cl->logoutUser();
+				}
+			}
+			break;
+
+		case Snapshot:
+			foreach( client * cl, _clients )
+			{
+				cl->snapshot();
+			}
+			break;
+
+		case PowerOn:
+			foreach( client * cl, _clients )
+			{
+				cl->powerOn();
+			}
+			break;
+
+		case Reboot:
+			if ( confirmReboot( _target ) )
+			{
+				foreach( client * cl, _clients )
+				{
+					cl->reboot();
+				}
+			}
+			break;
+
+		case PowerDown:
+			if ( confirmPowerDown( _target ) )
+			{
+				foreach( client * cl, _clients )
+				{
+					cl->powerDown();
+				}
+			}
+			break;
+
+		case ExecCmds:
+			{
+				QString cmds;
+				cmdInputDialog cmd_input_dialog( cmds );
+
+				if( cmd_input_dialog.exec() == QDialog::Accepted &&
+					!cmds.isEmpty() )
+				{
+					foreach( client * cl, _clients )
+					{
+						cl->execCmds( cmds );
+					}
+				}
+			}
+			break;
+
+		case RemoteScript:
+			{
+				QString script = dataExpanded( _clients );
+				foreach( client * cl, _clients )
+				{
+					cl->execCmds( script );
+				}
+				break;
+			}
+
+		case LocalScript:
+			{
+				QString script = dataExpanded( _clients );
+				QProcess::startDetached( script );
+				break;
+			}
+	}
+}
+
+
+
+
+QString clientAction::dataExpanded( QVector<client *> _clients ) const
+{
+	static QRegExp s_reITALC_HOSTS( "\\$ITALC_HOSTS\\b" );
+
+	QString script = data().toString();
+
+	if ( script.contains( s_reITALC_HOSTS ) )
+	{
+		QStringList hosts;
+		foreach ( client *cl, _clients )
+		{
+			hosts << cl->hostname();
+		}
+		script.replace( s_reITALC_HOSTS, hosts.join( " " ) );
+	}
+
+	return script;
+}
+
+
+
+
+void clientAction::process( QAction * _action, QVector<client *> _clients,
+			targetGroup _target )
+{
+	clientAction * action = dynamic_cast<clientAction *>( _action );
+	if ( action )
+	{
+		action->process( _clients, _target );
+	}
+}
+
+
+
+
+bool clientAction::confirmLogout( targetGroup _target ) const
+{
+	QString question = ( _target == VisibleClients ?
+		tr( "Are you sure want logout all users on all visible computers ?" ) :
+		tr( "Are you sure want logout all users on all selected computers ?" ) );
+	
+	return QMessageBox::question( NULL, tr( "Logout user" ),
+				question, QMessageBox::Yes, QMessageBox::No )
+		== QMessageBox::Yes;
+}
+
+
+
+
+bool clientAction::confirmReboot( targetGroup _target ) const
+{
+	QString question = ( _target == VisibleClients ?
+		tr( "Are you sure want to reboot all visible computers?" ) :
+		tr( "Are you sure want to reboot all selected computers?" ) );
+
+	return QMessageBox::question( NULL, tr( "Reboot computers" ),
+				question, QMessageBox::Yes, QMessageBox::No )
+		== QMessageBox::Yes;
+}
+
+
+
+
+bool clientAction::confirmPowerDown( targetGroup _target ) const
+{
+	QString question = ( _target == VisibleClients ?
+		tr( "Are you sure want to power down all visible computers?" ) :
+		tr( "Are you sure want to power down all selected computers?" ) );
+
+	return QMessageBox::question( NULL, tr( "Reboot computers" ),
+				question, QMessageBox::Yes, QMessageBox::No )
+		== QMessageBox::Yes;
+}
+
+
+
+
+clientMenu::clientMenu( const QString & _title, const QList<QAction *> _actions,
+			QWidget * _parent, const bool _fullMenu ) :
+	QMenu( _title, _parent )
+{
+	setIcon( scaledIcon( "client.png" ) );
+
+	foreach ( QAction * action, _actions )
+	{
+		clientAction * act = dynamic_cast<clientAction *>( action );
+		if ( act && act->flags( clientAction::FullMenu ) && ! _fullMenu )
+		{
+			continue;
+		}
+
+		addAction( action );
+	}
+}
+
+
+
+QMenu * clientMenu::createDefault( QWidget * _parent )
+{
+	QMenu * menu = new QMenu( _parent );
+
+	menu->addAction( new clientAction( clientAction::Overview,
+		scaledIcon( "overview_mode.png" ), tr( "Overview" ), menu ) );
+	menu->addAction( new clientAction( clientAction::FullscreenDemo,
+		scaledIcon( "fullscreen_demo.png" ), tr( "Fullscreen demo" ), menu ) );
+	menu->addAction( new clientAction( clientAction::WindowDemo,
+		scaledIcon( "window_demo.png" ), tr( "Window demo" ), menu ) );
+	menu->addAction( new clientAction(	clientAction::Locked,
+		scaledIcon( "locked.png" ), tr( "Locked display" ), menu ) );
+	menu->addSeparator();
+
+	menu->addAction( new clientAction( clientAction::ViewLive,
+		scaledIcon( "viewmag.png" ), tr( "View live" ), menu,
+		clientAction::FullMenu ) );
+	menu->addAction( new clientAction( clientAction::RemoteControl,
+		scaledIcon( "remote_control.png" ), tr( "Remote control" ), menu,
+		clientAction::FullMenu ) );
+	menu->addAction( new clientAction( clientAction::ClientDemo,
+		scaledIcon( "client_demo.png" ), tr( "Let student show demo" ), menu,
+		clientAction::FullMenu ) );
+
+	menu->addAction( new clientAction( clientAction::SendTextMessage,
+		scaledIcon( "text_message.png" ), tr( "Send text message" ), menu ) );
+	menu->addSeparator();
+
+	menu->addAction( new clientAction( clientAction::LogonUser,
+		scaledIcon( "remotelogon.png" ), tr( "Logon user" ), menu ) );
+	menu->addAction( new clientAction( clientAction::LogoutUser,
+		scaledIcon( "logout.png" ), tr( "Logout user" ), menu ) );
+	menu->addSeparator();
+
+	menu->addAction( new clientAction( clientAction::Snapshot,
+		scaledIcon( "snapshot.png" ), tr( "Take a snapshot" ), menu ) );
+	menu->addSeparator();
+
+	menu->addAction( new clientAction( clientAction::PowerOn,
+		scaledIcon( "power_on.png" ), tr( "Power on" ), menu ) );
+	menu->addAction( new clientAction( clientAction::Reboot,
+		scaledIcon( "reboot.png" ), tr( "Reboot" ), menu ) );
+	menu->addAction( new clientAction( clientAction::PowerDown,
+		scaledIcon( "power_off.png" ), tr( "Power down" ), menu ) );
+	menu->addAction( new clientAction( clientAction::ExecCmds,
+		scaledIcon( "run.png" ), tr( "Execute commands" ), menu ) );
+
+	return menu;
+}
+
+
+
+
+
+
+
+
+client::client( const QString & _hostname,
+		const QString & _mac, const QString & _nickname,
 		types _type, classRoom * _class_room,
 					mainWindow * _main_window, int _id ) :
 	QWidget( _main_window->workspace() ),
@@ -148,8 +473,8 @@ client::client( const QString & _local_ip,
 	m_connection( NULL ),
 	m_clickPoint( -1, -1 ),
 	m_origPos( -1, -1 ),
-	m_name( _name ),
-	m_localIP( _local_ip ),
+	m_hostname( _hostname ),
+	m_nickname( _nickname ),
 	m_mac( _mac ),
 	m_type( _type ),
 	m_reloadsAfterReset( 0 ),
@@ -159,7 +484,6 @@ client::client( const QString & _local_ip,
 	m_state( State_Unkown ),
 	m_syncMutex(),
 	m_classRoomItem( NULL ),
-	m_contextMenu( NULL ),
 	m_updateThread( NULL )
 {
 	new closeButton( this );
@@ -173,7 +497,7 @@ client::client( const QString & _local_ip,
 
 	m_classRoomItem = new classRoomItem( this, _class_room );
 
-	m_connection = new ivsConnection( m_localIP,
+	m_connection = new ivsConnection( m_hostname,
 						ivsConnection::QualityLow );
 
 	setAttribute( Qt::WA_OpaquePaintEvent );
@@ -197,36 +521,6 @@ client::client( const QString & _local_ip,
 	setFixedSize( DEFAULT_CLIENT_SIZE );
 	//resize( DEFAULT_CLIENT_SIZE );
 
-	QMenu * tb = new QMenu( this );
-	m_contextMenu = tb;
-	connect( tb, SIGNAL( triggered( QAction * ) ), this,
-					SLOT( processCmdSlot( QAction * ) ) );
-	tb->addAction( scaled( ":/resources/overview_mode.png", 16, 16 ),
-			tr( "Overview" ) )->
-					setData( Cmd_CmdCount + Mode_Overview );
-	tb->addAction( scaled( ":/resources/fullscreen_demo.png", 16, 16 ),
-						tr( "Fullscreen demo" ) )->
-				setData( Cmd_CmdCount + Mode_FullscreenDemo );
-	tb->addAction( scaled( ":/resources/window_demo.png", 16, 16 ),
-						tr( "Window demo" ) )->
-				setData( Cmd_CmdCount + Mode_WindowDemo );
-	tb->addAction( scaled( ":/resources/locked.png", 16, 16 ),
-						tr( "Locked display" ) )->
-				setData( Cmd_CmdCount + Mode_Locked );
-	tb->addSeparator();
-	for( int i = Cmd_ViewLive; i < Cmd_CmdCount; ++i )
-	{
-		QAction * a = tb->addAction( scaled(
-			QString( ":/resources/" ) + s_commands[i].m_icon,
-								16, 16 ),
-						tr( s_commands[i].m_name ) );
-		a->setData( i );
-		if( s_commands[i].m_insertSep )
-		{
-			tb->addSeparator();
-		}
-	}
-
 	m_updateThread = new updateThread( this );
 }
 
@@ -235,16 +529,27 @@ client::client( const QString & _local_ip,
 
 client::~client()
 {
-	changeMode( client::Mode_Overview, m_mainWindow->localISD() );
-	m_updateThread->quit();
-	m_updateThread->wait();
-	m_updateThread->update();
+	quit();
+
+	m_updateThread->wait(
+#ifdef BUILD_WIN32
+			2000
+#else
+			5000
+#endif
+				);
+	if( !m_updateThread->isFinished() )
+	{
+		localSystem::sleep( 500 );
+		m_updateThread->terminate();
+	}
+
 	delete m_updateThread;
 
-	m_syncMutex.lock();
+//	m_syncMutex.lock();
 	delete m_connection;
 	m_connection = NULL;
-	m_syncMutex.unlock();
+//	m_syncMutex.unlock();
 
 	delete m_classRoomItem;
 }
@@ -252,6 +557,15 @@ client::~client()
 
 
 
+void client::quit( void )
+{
+	changeMode( Mode_Overview );
+	m_updateThread->quit();
+}
+
+
+
+
 int client::id( void ) const
 {
 	QHash<int, client *>::const_iterator it;
@@ -295,8 +609,10 @@ int client::freeID( void )
 
 
 
-void client::changeMode( const modes _new_mode, isdConnection * _conn )
+void client::changeMode( const modes _new_mode )
 {
+	isdConnection * conn = m_mainWindow->localISD();
+
 	if( _new_mode != m_mode )
 	{
 		//m_syncMutex.lock();
@@ -307,7 +623,7 @@ void client::changeMode( const modes _new_mode, isdConnection * _conn )
 				break;
 			case Mode_FullscreenDemo:
 			case Mode_WindowDemo:
-				_conn->demoServerDenyClient( m_localIP );
+				conn->demoServerDenyClient( m_hostname );
 				m_updateThread->enqueueCommand(
 						updateThread::Cmd_StopDemo );
 				break;
@@ -323,13 +639,11 @@ void client::changeMode( const modes _new_mode, isdConnection * _conn )
 				break;
 			case Mode_FullscreenDemo:
 			case Mode_WindowDemo:
-				_conn->demoServerAllowClient( m_localIP );
+				conn->demoServerAllowClient( m_hostname );
 				m_updateThread->enqueueCommand(
 						updateThread::Cmd_StartDemo,
-	QList<QVariant>()
-			<< _conn->host() + ":" + QString::number(
-						_conn->demoServerPort() )
-			<< (int)( m_mode == Mode_FullscreenDemo ) );
+	QList<QVariant>() << QString::number( conn->demoServerPort() )
+				<< (int)( m_mode == Mode_FullscreenDemo ) );
 				//m_mode = Mode_FullscreenDemo;
 				break;
 			case Mode_Locked:
@@ -344,9 +658,9 @@ void client::changeMode( const modes _new_mode, isdConnection * _conn )
 	// why we offer this lines
 	else if( m_mode == Mode_Overview )
 	{
-		if( _conn != NULL )
+		if( conn != NULL )
 		{
-			_conn->demoServerDenyClient( m_localIP );
+			conn->demoServerDenyClient( m_hostname );
 		}
 		m_updateThread->enqueueCommand( updateThread::Cmd_StopDemo );
 		m_updateThread->enqueueCommand( updateThread::Cmd_UnlockScreen );
@@ -369,7 +683,7 @@ void client::setClassRoom( classRoom * _cr )
 void client::resetConnection( void )
 {
 	m_updateThread->enqueueCommand( updateThread::Cmd_ResetConnection,
-								m_localIP );
+								m_hostname );
 }
 
 
@@ -377,6 +691,21 @@ void client::resetConnection( void )
 
 void client::update( void )
 {
+	// at least set tooltip with user-name if it is not displayed
+	// in title-bar
+	if( m_connection->state() == ivsConnection::Connected &&
+		!m_mainWindow->getClassroomManager()->showUsername() )
+	{
+		if( toolTip() != m_user )
+		{
+			setToolTip( m_user );
+		}
+	}
+	else
+	{
+		setToolTip( QString() );
+	}
+
 	m_state = currentState();
 	QWidget::update();
 }
@@ -384,53 +713,12 @@ void client::update( void )
 
 
 
-void client::createActionMenu( QMenu * _m )
-{
-	connect( _m, SIGNAL( triggered( QAction * ) ), this,
-					SLOT( processCmdSlot( QAction * ) ) );
-	_m->addActions( m_contextMenu->actions() );
-}
-
-
-
-
-void client::processCmd( clientCmds _cmd, const QString & _u_data )
-{
-	if( _cmd < 0 || _cmd >= Cmd_CmdCount )
-	{
-		return;
-	}
-
-	( this->*( client::s_commands[_cmd].m_exec ) )( _u_data );
-}
-
-
-
-
-void client::processCmdSlot( QAction * _action )
-{
-	int a = _action->data().toInt();
-	if( a >= Cmd_ViewLive && a < Cmd_CmdCount )
-	{
-		processCmd( static_cast<clientCmds>( a ) );
-	}
-	else if( a >= Cmd_CmdCount+Mode_Overview &&
-						a < Cmd_CmdCount+Mode_Unknown )
-	{
-		changeMode( static_cast<modes>( a - Cmd_CmdCount ),
-						m_mainWindow->localISD() );
-	}
-}
-
-
-
-
 bool client::userLoggedIn( void )
 {
 	QMutexLocker ml( &m_syncMutex );
 
 	return( m_connection->state() == ivsConnection::Connected ||
-		m_connection->reset( m_localIP, &m_reloadsAfterReset ) ==
+		m_connection->reset( m_hostname, &m_reloadsAfterReset ) ==
 						ivsConnection::Connected );
 }
 
@@ -440,7 +728,7 @@ bool client::userLoggedIn( void )
 void client::closeEvent( QCloseEvent * _ce )
 {
 	// make sure, client isn't forgotten by teacher after being hidden
-	changeMode( Mode_Overview, m_mainWindow->localISD() );
+	changeMode( Mode_Overview );
 	hide();
 	_ce->ignore();
 }
@@ -450,7 +738,8 @@ void client::closeEvent( QCloseEvent * _ce )
 
 void client::contextMenuEvent( QContextMenuEvent * )
 {
-	m_contextMenu->exec( QCursor::pos() );
+	/* classRoomManager handles clientMenu */
+	m_mainWindow->getClassroomManager()->clientMenuRequest();
 }
 
 
@@ -463,7 +752,8 @@ void client::hideEvent( QHideEvent * )
 		hide();
 	}
 
-	if( m_classRoomItem != NULL )
+	if( m_classRoomItem != NULL && m_mainWindow->isVisible() &&
+						!m_mainWindow->isMinimized() )
 	{
 		m_classRoomItem->setVisible( FALSE );
 	}
@@ -472,18 +762,88 @@ void client::hideEvent( QHideEvent * )
 
 
 
+void client::enlarge()
+{
+	if ( m_origSize.isValid() ) 
+	{
+		QWidget * workspace = m_mainWindow->workspace();
+		QSize s = QSize( m_origSize );
+		
+		s.scale( workspace->parentWidget()->size(),
+							Qt::KeepAspectRatio );
+		setFixedSize( s );
+
+		/* centralize */
+		QSize offset = ( workspace->parentWidget()->size() - s ) / 2;
+		move( offset.width() - workspace->x(),
+			offset.height() - workspace->y() );
+
+		raise();
+	}
+}
+
+
+
+
+void client::zoom()
+{
+	m_origPos = pos();
+	m_origSize = size();
+	/* Delay zooming before we are sure that
+	 * this is not just a fast click.
+	 */
+	QTimer::singleShot( 300, this, SLOT( enlarge() ) );
+}
+
+
+
+
+void client::zoomBack()
+{
+	if ( m_origSize.isValid() )
+	{	
+		move( m_origPos );
+		setFixedSize( m_origSize );
+		/* reset value: */
+		m_origSize = QSize();
+	}
+}
+
+
+
+
 void client::mousePressEvent( QMouseEvent * _me )
 {
+	classTreeWidget * tree = static_cast<classTreeWidget *>(
+				m_classRoomItem->treeWidget() );
+
+	tree->setCurrentItem( m_classRoomItem );
+
 	if( _me->button() == Qt::LeftButton )
 	{
 		raise();
 		m_clickPoint = _me->globalPos();
 		m_origPos = pos();
+
+		if ( ! ( _me->modifiers() & ( Qt::ControlModifier | Qt::ShiftModifier ) ))
+		{
+			tree->clearSelection();
+		}
+		m_classRoomItem->setSelected( ! m_classRoomItem->isSelected() );
+
+		zoom();
+
+		_me->ignore();
 	}
-	else
+	else if ( _me->button() == Qt::RightButton )
 	{
-		QWidget::mousePressEvent( _me );
+		if ( ! m_classRoomItem->isSelected() ) {
+			tree->clearSelection();
+			m_classRoomItem->setSelected( TRUE );
+		}
 	}
+
+	QWidget::mousePressEvent( _me );
 }
 
 
@@ -493,6 +853,7 @@ void client::mouseMoveEvent( QMouseEvent * _me )
 {
 	if( m_clickPoint.x() >= 0 )
 	{
+		zoomBack();
 		move( m_origPos + _me->globalPos() - m_clickPoint );
 		parentWidget()->updateGeometry();
 	}
@@ -507,6 +868,7 @@ void client::mouseMoveEvent( QMouseEvent * _me )
 
 void client::mouseReleaseEvent( QMouseEvent * _me )
 {
+	zoomBack();
 	m_clickPoint = QPoint( -1, -1 );
 	QWidget::mouseReleaseEvent( _me );
 }
@@ -518,11 +880,11 @@ void client::mouseDoubleClickEvent( QMouseEvent * _me )
 {
 	if( m_mainWindow->getClassroomManager()->clientDblClickAction() == 0 )
 	{
-		remoteControl( "" );
+		remoteControl();
 	}
 	else
 	{
-		viewLive( "" );
+		viewLive();
 	}
 }
 
@@ -554,26 +916,28 @@ void client::paintEvent( QPaintEvent * _pe )
 	p.drawRect( QRect( 0, 0, width()-1, height()-1 ) );
 	p.setRenderHint( QPainter::Antialiasing, TRUE );
 
-	const QString s = m_user != "" ? m_user :
-		( m_name + " (" + m_classRoomItem->parent()->text( 0 ) +
+	p.fillRect( 1, 1, width()-2, TITLE_HEIGHT-2,
+			m_classRoomItem->isSelected() ?
+				QColor( 96, 96, 96 ) :
+						QColor( 224, 224, 224 ) );
+
+	bool showUsername = m_mainWindow->getClassroomManager()->showUsername();
+	const QString s = (showUsername && m_user != "") ? m_user :
+		( name() + " (" + m_classRoomItem->parent()->text( 0 ) +
 									")" );
 	QFont f = p.font();
 	f.setBold( TRUE );
 	p.setFont( f );
-	p.setPen( Qt::gray );
-	p.drawText( 11, TITLE_HEIGHT-3, s );
-	p.setPen( Qt::black );
-	p.drawText( 10, TITLE_HEIGHT-4, s );
+	p.setPen( m_classRoomItem->isSelected() ? Qt::white : Qt::black );
+	p.drawText( 10, TITLE_HEIGHT-7, s );
 
 	if( m_connection->state() == ivsConnection::Connected &&
 						m_mode == Mode_Overview )
 	{
-		p.drawImage( CONTENT_OFFSET, m_connection->scaledScreen());
+		p.drawImage( CONTENT_OFFSET, m_connection->scaledScreen() );
 	}
 	else
 	{
-	/*	p.drawImage( _pe->rect().topLeft() + CONTENT_OFFSET,
-						m_statePixmap, _pe->rect() );*/
 	const int aw = width() - 2*DECO_WIDTH;
 	const int ah = height() - CONTENT_SIZE_SUB.height() - DECO_WIDTH;
 
@@ -639,7 +1003,7 @@ void client::paintEvent( QPaintEvent * _pe )
 
 void client::resizeEvent( QResizeEvent * _re )
 {
-	findChildren<closeButton*>()[0]->move( width()-21, 4 );
+	findChild<closeButton*>()->move( width()-21, 3 );
 	m_connection->setScaledSize( size() - CONTENT_SIZE_SUB );
 	m_connection->rescaleScreen();
 	QWidget::resizeEvent( _re );
@@ -660,7 +1024,7 @@ void client::showEvent( QShowEvent * )
 
 
 
-void client::reload( const QString & _update )
+void client::reload()
 {
 	if( userLoggedIn() )
 	{
@@ -693,13 +1057,6 @@ void client::reload( const QString & _update )
 		m_user = "";
 	}
 
-
-	// if we are called out of main-thread, we may update...
-	if( _update == CONFIRM_YES )
-	{
-		update();
-	}
-
 	if( m_classRoomItem )
 	{
 		m_classRoomItem->setUser( m_user );
@@ -709,45 +1066,44 @@ void client::reload( const QString & _update )
 
 
 
-void client::clientDemo( const QString & )
+void client::clientDemo()
 {
 	classroomManager * cm = m_mainWindow->getClassroomManager();
 	cm->changeGlobalClientMode( Mode_Overview );
 
-	isdConnection * conn = m_mainWindow->localISD();
 	QVector<client *> vc = cm->visibleClients();
 
 	foreach( client * cl, vc )
 	{
 		if( cl != this )
 		{
-			cl->changeMode( Mode_FullscreenDemo, conn );
+			cl->changeMode( Mode_FullscreenDemo );
 		}
 	}
 
 	//m_mainWindow->checkModeButton( client::Mode_FullscreenDemo );
 
-	m_mainWindow->remoteControlDisplay( m_localIP, TRUE, TRUE );
+	m_mainWindow->remoteControlDisplay( m_hostname, TRUE, TRUE );
 }
 
 
 
 
-void client::viewLive( const QString & )
+void client::viewLive()
 {
-	changeMode( Mode_Overview, m_mainWindow->localISD() );
+	changeMode( Mode_Overview );
 
-	m_mainWindow->remoteControlDisplay( m_localIP, TRUE );
+	m_mainWindow->remoteControlDisplay( m_hostname, TRUE );
 }
 
 
 
 
-void client::remoteControl( const QString & )
+void client::remoteControl()
 {
-	changeMode( Mode_Overview, m_mainWindow->localISD() );
+	changeMode( Mode_Overview );
 
-	m_mainWindow->remoteControlDisplay( m_localIP );
+	m_mainWindow->remoteControlDisplay( m_hostname );
 }
 
 
@@ -755,50 +1111,24 @@ void client::remoteControl( const QString & )
 
 void client::sendTextMessage( const QString & _msg )
 {
-	if( _msg.isEmpty() )
-	{
-		QString msg;
-
-		textMessageDialog tmd( msg, this );
-		if( tmd.exec() == QDialog::Accepted && msg != "" )
-		{
-			sendTextMessage( msg );
-		}
-	}
-	else
-	{
-		m_updateThread->enqueueCommand(
-				updateThread::Cmd_SendTextMessage, _msg );
-	}
+	m_updateThread->enqueueCommand(
+			updateThread::Cmd_SendTextMessage, _msg );
 }
 
 
 
 
-void client::logonUser( const QString & _uname_and_pw )
+void client::logonUser( const QString & _username, const QString & _password,
+			const QString & _domain )
 {
-
-	if( _uname_and_pw.isEmpty() )
-	{
-		remoteLogonDialog mld( this );
-		if( mld.exec() == QDialog::Accepted &&
-			!mld.userName().isEmpty() && !mld.password().isEmpty() )
-		{
-			logonUser( mld.userName() + "*" + mld.password() +
-							"*" + mld.domain() );
-		}
-	}
-	else
-	{
-		m_updateThread->enqueueCommand( updateThread::Cmd_LogonUser,
-								_uname_and_pw );
-	}
+	m_updateThread->enqueueCommand( updateThread::Cmd_LogonUser,
+		_username + "*" + _password + "*" + _domain );
 }
 
 
 
 
-void client::logoutUser( const QString & _confirm )
+void client::logoutUser()
 {
 	m_updateThread->enqueueCommand( updateThread::Cmd_LogoutUser );
 }
@@ -806,7 +1136,7 @@ void client::logoutUser( const QString & _confirm )
 
 
 
-void client::snapshot( const QString & )
+void client::snapshot()
 {
 	m_makeSnapshot = TRUE;
 }
@@ -815,7 +1145,7 @@ void client::snapshot( const QString & )
 
 
 
-void client::powerOn( const QString & )
+void client::powerOn()
 {
 	m_mainWindow->localISD()->wakeOtherComputer( m_mac );
 }
@@ -823,7 +1153,7 @@ void client::powerOn( const QString & )
 
 
 
-void client::reboot( const QString & _confirm )
+void client::reboot()
 {
 	m_updateThread->enqueueCommand( updateThread::Cmd_Reboot );
 }
@@ -832,7 +1162,7 @@ void client::reboot( const QString & _confirm )
 
 
 
-void client::powerDown( const QString & _confirm )
+void client::powerDown()
 {
 	m_updateThread->enqueueCommand( updateThread::Cmd_PowerDown );
 }
@@ -842,22 +1172,7 @@ void client::powerDown( const QString & _confirm )
 
 void client::execCmds( const QString & _cmds )
 {
-	if( _cmds.isEmpty() )
-	{
-		QString cmds;
-
-		cmdInputDialog cmd_input_dialog( cmds, this );
-		if( cmd_input_dialog.exec() == QDialog::Accepted &&
-							!cmds.isEmpty() )
-		{
-			execCmds( cmds );
-		}
-	}
-	else
-	{
-		m_updateThread->enqueueCommand( updateThread::Cmd_ExecCmds,
-									_cmds );
-	}
+	m_updateThread->enqueueCommand( updateThread::Cmd_ExecCmds, _cmds );
 }
 
 
@@ -980,15 +1295,16 @@ void updateThread::update( void )
 	m_client->m_syncMutex.unlock();
 	m_queueMutex.unlock();
 
-	if( m_client->isVisible() )
+	if( m_client->m_classRoomItem->isVisible() )
 	{
 		if( !m_client->m_mainWindow->remoteControlRunning() &&
 						!mainWindow::atExit() )
 		{
-			m_client->processCmd( client::Cmd_Reload, CONFIRM_NO );
+			m_client->reload();
 		}
 	}
-	else if( m_client->m_connection->state() == ivsConnection::Connected )
+	else if( m_client->m_connection->state() == ivsConnection::Connected
+						&& !mainWindow::atExit() )
 	{
 		m_client->m_connection->close();
 	}
