@@ -38,12 +38,11 @@
 
 QSplashScreen * splashScreen = NULL;
 
-QString __demo_network_interface;
-QString __demo_master_ip;
 QString __default_domain;
 int __demo_quality = 0;
 
 int __isd_port = PortOffsetISD;
+QString __isd_host = "127.0.0.1";
 
 
 // good old main-function... initializes qt-app and starts iTALC
@@ -52,8 +51,30 @@ int main( int argc, char * * argv )
 	QApplication app( argc, argv );
 	app.connect( &app, SIGNAL( lastWindowClosed() ), SLOT( quit() ) );
 
+#if QT_VERSION >= 0x040300
+	app.setStyleSheet(
+		"QMenu { border:1px solid black; background-color: white; "
+			"background-image:url(:/resources/tray-menu-bg.png); "
+			"background-repeat:no-repeat; "
+			"background-position: bottom right; }"
+		"QMenu::separator { height: 1px; background: rgb(128,128,128); "
+					"margin-left: 5px; margin-right: 5px; }"
+		"QMenu::item { padding: 2px 32px 2px 20px; "
+						"margin:3px; }"
+		"QMenu::item:selected { color: white; font-weight:bold; "
+			"background-color: rgba(0, 0, 0, 160); "
+						"margin:3px; }"
+		"QMenu::item:disabled { color: white;  margin:0px; "
+			"background-color: rgba(0,0,0,192); font-size:14px;"
+			"font-weight:bold; padding: 4px 32px 4px 20px; }" );
+#endif
+
 	// load translations
-	const QString loc = QLocale::system().name().left( 2 );
+	QString loc =  QLocale::system().name().toLower();
+	if( loc.left( 2 ) == loc.right( 2 ) )
+	{
+		loc = loc.left( 2 );
+	}
 
 	QTranslator app_tr;
 	app_tr.load( ":/resources/" + loc + ".qm" );
@@ -75,6 +96,10 @@ int main( int argc, char * * argv )
 	localSystem::initialize();
 
 	__role = ISD::RoleTeacher;
+	if( localSystem::parameter( "isdport" ).toInt() > 0 )
+	{
+		__isd_port = localSystem::parameter( "isdport" ).toInt();
+	}
 
 	// parse arguments
 	QStringListIterator arg_it( QCoreApplication::arguments() );
@@ -128,6 +153,11 @@ int main( int argc, char * * argv )
 		{
 			__isd_port = arg_it.next().toInt();
 		}
+		else if( a == "-isdhost" && arg_it.hasNext() )
+		{
+			__isd_host = arg_it.next();
+		}
+
 	}
 
 
