@@ -30,8 +30,10 @@
 #include <QtCore/QThread>
 #include <QtGui/QButtonGroup>
 #include <QtGui/QMainWindow>
+#include <QtGui/QSystemTrayIcon>
 #include <QtGui/QToolButton>
 
+#include "client.h"
 #include "snapshot_list.h"
 
 
@@ -42,15 +44,12 @@ class QSplitter;
 class classroomManager;
 class configWidget;
 class toolBar;
-class supportWidget;
 class italcSideBar;
 class isdConnection;
 class overviewWidget;
 class remoteControlWidget;
 class userList;
 
-extern QString __demo_network_interface;
-extern QString __demo_master_ip;
 extern QString __default_domain;
 extern int __demo_quality;
 
@@ -85,7 +84,13 @@ public:
 
 	virtual QSize sizeHint( void ) const;
 
+
 private:
+	virtual void contextMenuEvent( QContextMenuEvent * _event );
+
+	QMenu * m_contextMenu;
+
+	friend class mainWindow;
 
 } ;
 
@@ -123,6 +128,16 @@ public:
 		}
 	}
 
+	inline toolBar * getToolBar( void )
+	{
+		return( m_toolBar );
+	}
+
+	inline italcSideBar * getSideBar( void )
+	{
+		return( m_sideBar );
+	}
+
 	static bool ensureConfigPathExists( void );
 
 	static inline bool atExit( void )
@@ -130,7 +145,7 @@ public:
 		return( s_atExit );
 	}
 
-	void remoteControlDisplay( const QString & _ip,
+	void remoteControlDisplay( const QString & _hostname,
 					bool _view_only = FALSE,
 					bool _stop_demo_afterwards = FALSE );
 
@@ -151,12 +166,31 @@ protected:
 
 
 private slots:
+	void handleSystemTrayEvent( QSystemTrayIcon::ActivationReason _r );
+	void remoteControlClient( QAction * _c );
 	void remoteControlWidgetClosed( QObject * );
 
 	void aboutITALC( void );
 
 	void changeGlobalClientMode( int );
 
+	void mapOverview( void )
+	{
+		changeGlobalClientMode( client::Mode_Overview );
+	}
+	void mapFullscreenDemo( void )
+	{
+		changeGlobalClientMode( client::Mode_FullscreenDemo );
+	}
+	void mapWindowDemo( void )
+	{
+		changeGlobalClientMode( client::Mode_WindowDemo );
+	}
+	void mapScreenLock( void )
+	{
+		changeGlobalClientMode( client::Mode_Locked );
+	}
+
 
 private:
 	virtual void closeEvent( QCloseEvent * _ce );
@@ -169,6 +203,8 @@ private:
 
 	toolBar * m_toolBar;
 
+	QList<QAction *> m_sysTrayActions;
+
 	QSplitter * m_splitter;
 
 	QWidget * m_sideBarWidget;
@@ -189,7 +225,6 @@ private:
 	userList * m_userList;
 	snapshotList * m_snapshotList;
 	configWidget * m_configWidget;
-	supportWidget * m_supportWidget;
 
 	static bool s_atExit;
 
