@@ -1,7 +1,7 @@
 /*
  * tool_button.h - declaration of class toolButton 
  *
- * Copyright (c) 2006-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2006-2008 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of iTALC - http://italc.sourceforge.net
  *
@@ -31,6 +31,9 @@
 #include "fast_qimage.h"
 
 
+class QToolBar;
+
+
 class toolButton : public QToolButton
 {
 	Q_OBJECT
@@ -40,9 +43,20 @@ public:
 				const QString & _title, 
 				const QString & _desc, QObject * _receiver, 
 				const char * _slot, QWidget * _parent );
+	toolButton( QAction * _a, const QString & _label,
+				const QString & _alt_label,
+				const QString & _desc, QObject * _receiver, 
+				const char * _slot, QWidget * _parent );
 	virtual ~toolButton();
 
 
+	static void setIconOnlyMode( bool _enabled );
+
+	static bool iconOnlyMode( void )
+	{
+		return( s_iconOnlyMode );
+	}
+
 	static void setToolTipsDisabled( bool _disabled )
 	{
 		s_toolTipsDisabled = _disabled;
@@ -53,6 +67,7 @@ public:
 		return( s_toolTipsDisabled );
 	}
 
+	void addTo( QToolBar * );
 
 
 protected:
@@ -67,11 +82,16 @@ signals:
 
 
 private slots:
+	bool checkForLeaveEvent( void );
 	void updateColorLevel( void );
 
 
 private:
+	void updateSize( void );
+
+
 	static bool s_toolTipsDisabled;
+	static bool s_iconOnlyMode;
 
 	QPixmap m_pixmap;
 	fastQImage m_img;
