@@ -86,36 +86,17 @@ bool isdConnection::initAuthentication( void )
 		qWarning( "isdConnection::initAuthentication(): private key "
 							"already initialized" );
 		delete privDSAKey;
+		privDSAKey = NULL;
 	}
 
 	const QString priv_key_file = localSystem::privateKeyPath( __role );
-
+	if( priv_key_file == "" )
+	{
+		return( FALSE );
+	}
 	privDSAKey = new privateDSAKey( priv_key_file );
 
 	return( privDSAKey->isValid() );
-
-/*	// key valid (i.e. could be loaded)?
-	if( !privDSAKey->isValid() )
-	{
-		// no, then create a new one
-		delete privDSAKey;
-		// generate 1024bit RSA-key and save it
-		privDSAKey = new privateDSAKey( 1024 );
-		if( !privDSAKey->isValid() )
-		{
-			qCritical( "isdConnection::initAuthentication(): "
-						"key generation failed!" );
-			return( FALSE );
-		}
-		privDSAKey->save( priv_key_file );
-
-		// derive public key from private key and save it
-		publicDSAKey( *privDSAKey ).save(
-					localSystem::publicKeyPath( __role ) );
-
-		return( FALSE );
-	}
-	return( TRUE );*/
 }
 
 
@@ -327,16 +308,19 @@ isdConnection::states isdConnection::open( void )
 			m_socket->error() == QTcpSocket::NetworkError )
 #endif
 	{
+#ifdef BUILD_WIN32
+		localSystem::sleep( 2000 );
+#endif
 		return( m_state = HostUnreachable );
 	}
 
 #ifndef NO_QTCPSOCKET_CONNECT
-	m_socket->waitForConnected( 3000 );
+	m_socket->waitForConnected( 5000 );
 #endif
 
 	if( m_socket->state() != QTcpSocket::ConnectedState )
 	{
-		qWarning( "isdConnection::open(): unable to connect to server "
+		qDebug( "isdConnection::open(): unable to connect to server "
                                "on client %s:%i", m_host.toAscii().constData(),
 								m_port );
 		if( m_socket->error() == QTcpSocket::ConnectionRefusedError )
@@ -379,6 +363,20 @@ void isdConnection::close( void )
 
 
 
+void isdConnection::gracefulClose( void )
+{
+	if( m_socket != NULL )
+	{
+		m_socket->close();
+		delete m_socket;
+		m_socket = NULL;
+	}
+	close();
+}
+
+
+
+
 isdConnection::states isdConnection::reset( const QString & _host, int * _cnt )
 {
 	close();
@@ -414,6 +412,7 @@ bool isdConnection::readFromServer( char * _out, const unsigned int _n )
 	}
 
 	unsigned int bytes_done = 0;
+	int tries = 0;
 
 	while( bytes_done < _n )
 	{
@@ -431,7 +430,8 @@ bool isdConnection::readFromServer( char * _out, const unsigned int _n )
 		{
 			// could not read data because we're not connected
 			// anymore?
-			if( m_socket->state() != QTcpSocket::ConnectedState )
+			if( m_socket->state() != QTcpSocket::ConnectedState ||
+								++tries > 400 )
 			{
 				qWarning( "isdConnection::readFromServer(): "
 						"connection failed: %d",
@@ -441,7 +441,11 @@ bool isdConnection::readFromServer( char * _out, const unsigned int _n )
 			}
 			m_socket->waitForReadyRead( 50 );
 		}
-		bytes_done += bytes_read;
+		else
+		{
+			bytes_done += bytes_read;
+			tries /= 2;
+		}
 	}
 
 	return( TRUE );
@@ -475,7 +479,7 @@ bool isdConnection::writeToServer( const char * _buf, const unsigned int _n )
 		bytes_done += bytes_written;
 	}
 
-	return( m_socket->waitForBytesWritten( 50 ) );
+	return( m_socket->waitForBytesWritten( 100 ) );
 }
 
 
@@ -754,7 +758,7 @@ bool isdConnection::execCmds( const QString & _cmd )
 
 
 
-bool isdConnection::startDemo( const QString & _master_ip, bool _full_screen )
+bool isdConnection::startDemo( const QString & _port, bool _full_screen )
 {
 	if( m_socket == NULL ||
 			m_socket->state() != QTcpSocket::ConnectedState )
@@ -764,7 +768,7 @@ bool isdConnection::startDemo( const QString & _master_ip, bool _full_screen )
 	}
 	return( ISD::msg( &m_socketDev, _full_screen ?
 			ISD::StartFullScreenDemo : ISD::StartWindowDemo ).
-					addArg( "ip", _master_ip ).send() );
+					addArg( "port", _port ).send() );
 }
 
 
@@ -874,26 +878,6 @@ bool isdConnection::collectFiles( const QString & _nfilter )
 
 
 
-/*
-bool isdConnection::remoteControlDisplay( const QString & _ip, bool _view_only )
-{
-	if( m_socket == NULL ||
-			m_socket->state() != QTcpSocket::ConnectedState )
-	{
-		m_state = Disconnected;
-		return( FALSE );
-	}
-	if( _view_only )
-	{
-		return( ISD::msg( &m_socketDev, ISD::ViewRemoteDisplay ).
-						addArg( "ip", _ip ).send() );
-	}
-	return( ISD::msg( &m_socketDev, ISD::RemoteControlDisplay ).
-						addArg( "ip", _ip ).send() );
-}
-*/
-
-
 
 bool isdConnection::wakeOtherComputer( const QString & _mac )
 {
@@ -985,20 +969,6 @@ bool isdConnection::demoServerRun( int _quality, int _port )
 
 
 
-bool isdConnection::demoServerStop( void )
-{
-	if( m_socket == NULL ||
-			m_socket->state() != QTcpSocket::ConnectedState )
-	{
-		m_state = Disconnected;
-		return( FALSE );
-	}
-	return( ISD::msg( &m_socketDev, ISD::DemoServer_Stop ).send() );
-}
-
-
-
-
 bool isdConnection::demoServerAllowClient( const QString & _client )
 {
 	if( m_socket == NULL ||
@@ -1029,6 +999,20 @@ bool isdConnection::demoServerDenyClient( const QString & _client )
 
 
 
+bool isdConnection::hideTrayIcon( void )
+{
+	if( m_socket == NULL ||
+			m_socket->state() != QTcpSocket::ConnectedState )
+	{
+		m_state = Disconnected;
+		return( FALSE );
+	}
+	return( ISD::msg( &m_socketDev, ISD::HideTrayIcon ).send() );
+}
+
+
+
+
 bool isdConnection::handleServerMessages( void )
 {
 	while( hasData() )
