@@ -34,6 +34,7 @@
 #include <QtCore/QMutex>
 #include <QtCore/QProcess>
 #include <QtCore/QSettings>
+#include <QtCore/QDateTime>
 #include <QtGui/QWidget>
 #include <QtNetwork/QTcpServer>
 
@@ -104,6 +105,7 @@ QString windowsConfigPath( int _type )
 #endif
 
 #include "local_system.h"
+#include "minilzo.h"
 
 
 
@@ -142,6 +144,13 @@ void msgHandler( QtMsgType _type, const char * _msg )
 	{
 		return ;
 	}
+#ifdef BUILD_WIN32
+	if( QString( _msg ).contains( "timers cannot be stopped",
+							Qt::CaseInsensitive ) )
+	{
+		exit( 0 );
+	}
+#endif
 	if( __debug_out == NULL )
 	{
 		QString tmp_path = QDir::rootPath() +
@@ -187,28 +196,28 @@ void msgHandler( QtMsgType _type, const char * _msg )
 		case QtDebugMsg:
 			if( localSystem::logLevel > 8)
 			{
-				out = QString( "[debug] %1" ).arg( _msg );
+				out = QDateTime::currentDateTime().toString() + QString( ": [debug] %1" ).arg( _msg ) + "\n";
 			}
 			break;
 		case QtWarningMsg:
 			if( localSystem::logLevel > 5 )
 			{
-				out = QString( "[warning] %1" ).arg( _msg );
+				out = QDateTime::currentDateTime().toString() + QString( ": [warning] %1" ).arg( _msg ) + "\n";
 			}
 			break;
 		case QtCriticalMsg:
 			if( localSystem::logLevel > 3 )
 			{
-				out = QString( "[critical] %1" ).arg( _msg );
+				out = QDateTime::currentDateTime().toString() + QString( ": [critical] %1" ).arg( _msg ) + "\n";
 			}
 			break;
 		case QtFatalMsg:
 			if( localSystem::logLevel > 1 )
 			{
-				out = QString( "[fatal] %1" ).arg( _msg );
+				out = QDateTime::currentDateTime().toString() + QString( ": [fatal] %1" ).arg( _msg ) + "\n";
 			}
 		default:
-			out = QString( "[unknown] %1" ).arg( _msg );
+			out = QDateTime::currentDateTime().toString() + QString( ": [unknown] %1" ).arg( _msg ) + "\n";
 			break;
 	}
 	if( out.trimmed().size() )
@@ -236,17 +245,18 @@ void initialize( p_pressKey _pk, const QString & _log_file )
 	__pressKey = _pk;
 	__log_file = _log_file;
 
+	lzo_init();
+
 	QCoreApplication::setOrganizationName( "iTALC Solutions" );
 	QCoreApplication::setOrganizationDomain( "italcsolutions.org" );
 	QCoreApplication::setApplicationName( "iTALC" );
 
 	QSettings settings( QSettings::SystemScope, "iTALC Solutions", "iTALC" );
 	
-	if( settings.contains( "settings/LogLevel" ) && 
-		   settings.value( "settings/LogLevel" ).toInt() != 0 )
+	if( settings.contains( "settings/LogLevel" ) )
 	{
 		logLevel = settings.value( "settings/LogLevel" ).toInt();
-	}	
+	}
 
 	qInstallMsgHandler( msgHandler );
 
@@ -256,9 +266,13 @@ void initialize( p_pressKey _pk, const QString & _log_file )
 
 
 
-int freePort( void )
+int freePort( int _default_port )
 {
 	QTcpServer t;
+	if( t.listen( QHostAddress::LocalHost, _default_port ) )
+	{
+		return( _default_port );
+	}
 	t.listen( QHostAddress::LocalHost );
 	return( t.serverPort() );
 }
@@ -426,6 +440,21 @@ void logonUser( const QString & _uname, const QString & _passwd,
 	{
 		return;
 	}
+/*
+ 	// does not work :(
+	enablePrivilege( SE_TCB_NAME, TRUE );
+	HANDLE hToken;
+	if( !LogonUser(
+		(CHAR *)_uname.toAscii().constData(),
+		(CHAR*)(_domain.isEmpty() ? "." : _domain.toAscii().constData()),
+		(CHAR *)_passwd.toAscii().constData(),
+		LOGON32_LOGON_INTERACTIVE,
+		LOGON32_PROVIDER_DEFAULT,
+		&hToken ) ) 
+	{
+		CloseHandle( hToken );
+	}
+	enablePrivilege( SE_TCB_NAME, FALSE );*/
 
 	// disable caps lock
 	if( GetKeyState( VK_CAPITAL ) & 1 )
